//These are the UART-Routines for the AVR-SoC project by Mario Mauerer.

#include <avr/io.h>
#include "uart.h"
#include "AVR_Softcore_main.h"

//This routine Initializes the UART16550 (from opencores.org):
void uart_init(void)
{

	LCR = 0b10000011; //stop bit: 1, 8 data bits, no parity, break disabled, no divisor latch access, access to divisors.

	DIVLATCHMSB = 0;
	DIVLATCHLSB = 16; //115200 Baud, the Core runs at 30MHz
	
	LCR = 0b00000011; //no access to divisor latches anymore
		
	FCR = 0xff; //fifo interrupt trigger level: 14

	MCR = 0x00; 

}

void uart_sendbyte(uint8_t data) //send a single byte to the Uart-FiFo.
{
	while(!(LSR & (1<<5))){}//wait until transmitter FIFO is empty.
	THR = data;

}

void uart_printstring(char *text) //print an entire string to the uart.
{
	while(*text)
	{
		while(!(LSR & (1<<5))){}//wait until transmitter FIFO is empty.
		//Now, as the FIFO is 14 Bytes wide, transfer 14 bytes directly to the FIFO:
		
		uint8_t count = 0;

		while(count<14 && *text!=0)
		{
			THR = *text; 
			text++;
			count++;	
		}

	}
	while(!(LSR & (1<<5))){}
	THR = 0x0D; //carriage return
	THR = 0x0A; //Line Feed	
}

uint8_t uart_receive() //receive a byte from the UART.
{	
	while(!(LSR & (1<<0))){}//wait until character arrives in fifo
		return RECBUFF;
}
